@@ -6,6 +6,7 @@ import subprocess
 import sys
 from io import StringIO
 from os import environ, getenv, path
+from uuid import uuid4
 
 import pkg_resources
 import yaml
@@ -37,7 +38,17 @@ class RealShellExecutor:
   def execute(self, commandWithArgs, envVars):
     print(
         f'➡️   Executing "{" ".join(commandWithArgs)}" (additional env vars: {" ".join(envVars.keys())})', file=sys.stderr)
-    return subprocess.run(commandWithArgs, env={**environ, **envVars})
+    return subprocess.run(commandWithArgs, env={**environ, **envVars}).returncode
+
+  def get_output(self, commandWithArgs):
+    cmdResult = subprocess.run(commandWithArgs, stdout=subprocess.PIPE, encoding='utf-8')
+    if cmdResult.returncode == 0:
+      return cmdResult.stdout
+    else:
+      return None
+
+  def is_tty(self):
+    return sys.stdout.isatty()
 
 
 class RealEnvAccessor:
@@ -57,6 +68,9 @@ class RealPrinter:
     else:
       print(message, file=sys.stdout)
 
+class TagGenerator:
+  def tag(self):
+    return str(uuid4()).replace('-', '')
 
 def _load_template(file, variables, template_accessor):
   content = template_accessor.load_template(file)
@@ -262,7 +276,7 @@ class Kubedev:
   def generate_tiltfile(self, projectName, images, portForwards, file_accessor, overwrite):
     print('💫 Generating Tiltfile...')
     tiltfile = StringIO()
-    for imageKey, image in images.items():
+    for _, image in images.items():
       tiltfile.write(f"docker_build('{image['imageNameTagless']}', '{image['buildPath']}')\n")
     tiltfile.write('\n')
 
@@ -278,11 +292,11 @@ class Kubedev:
     file_accessor.save_file('Tiltfile', tiltfile.getvalue(), overwrite)
 
   def generate_projects(self, images, file_accessor):
-    for imageKey, imageInfos in images.items():
-      print(f'🐳 Generating {imageKey}/Dockerfile...')
+    for _, imageInfos in images.items():
       path = imageInfos["buildPath"]
-      file_accessor.mkdirhier(path)
       dockerfile = f'{path}Dockerfile'
+      print(f'🐳 Generating {dockerfile}...')
+      file_accessor.mkdirhier(path)
       file_accessor.save_file(dockerfile, 'FROM scratch\n', False)
 
   def _get_kubecontext_arg(self, env_accessor):
@@ -301,7 +315,7 @@ class Kubedev:
         f'--set KUBEDEV_TAG="{tag}"' +
         KubedevConfig.get_helm_set_env_args(kubedev)
     ]
-    shell_executor.execute(command, variables)
+    return shell_executor.execute(command, variables)
 
   def template(self, configFileName, shell_executor=RealShellExecutor(), env_accessor=RealEnvAccessor(), file_accessor=RealFileAccessor()):
     return self.template_from_config(
@@ -315,27 +329,32 @@ class Kubedev:
         self._load_config(configFileName), shell_executor, env_accessor, file_accessor)
 
   def deploy_from_config(self, kubedev, shell_executor, env_accessor, file_accessor):
-    return self._template_or_deploy(kubedev, f"upgrade {kubedev['name']} ./helm-chart/ --install --wait", shell_executor, env_accessor, file_accessor)
+    release_name = KubedevConfig.get_helm_release_name(kubedev)
+    return self._template_or_deploy(kubedev, f"upgrade {release_name} ./helm-chart/ --install --wait", shell_executor, env_accessor, file_accessor)
 
   def build(self, configFileName, container, shell_executor=RealShellExecutor(), env_accessor=RealEnvAccessor()):
     return self.build_from_config(
-        self._load_config(configFileName), container=container, shell_executor=shell_executor, env_accessor=env_accessor)
+        self._load_config(configFileName), container=container, force_tag=None, shell_executor=shell_executor, env_accessor=env_accessor)
 
-  def build_from_config(self, kubedev, container, shell_executor, env_accessor):
+  def build_from_config(self, kubedev, container, force_tag, shell_executor, env_accessor):
     images = KubedevConfig.get_images(kubedev, env_accessor)
     if not container in images:
       raise KeyError(
           f"Container {container} is not defined in kubedev config.")
     else:
       image = images[container]
+      if force_tag is None:
+        imageTag = image['imageName']
+      else:
+        imageTag = f"{image['imageNameTagless']}:{force_tag}"
       call = [
           '/bin/sh',
           '-c',
-          f"docker build -t {image['imageName']} " +
+          f"docker build -t {imageTag} " +
           KubedevConfig.get_docker_build_args(image) +
           f"{image['buildPath']}"
       ]
-      shell_executor.execute(call, dict())
+      return shell_executor.execute(call, dict())
 
   def push(self, configFileName, container, shell_executor=RealShellExecutor(), env_accessor=RealEnvAccessor()):
     return self.push_from_config(
@@ -353,7 +372,7 @@ class Kubedev:
           '-c',
           f"docker push {image['imageName']}"
       ]
-      shell_executor.execute(call, dict())
+      return shell_executor.execute(call, dict())
 
   def check(self, configFileName, commands, env_accessor=RealEnvAccessor(), printer=RealPrinter(), file_accessor=RealFileAccessor()):
     return self.check_from_config(
@@ -395,3 +414,41 @@ class Kubedev:
     else:
       print('❌ Check failed')
     return result
+
+  def run(self, configFileName, container, env_accessor=RealEnvAccessor(), shell_executor=RealShellExecutor(), printer=RealPrinter(), file_accessor=RealFileAccessor()):
+    return self.run_from_config(
+        self._load_config(configFileName, file_accessor), container, env_accessor=env_accessor, printer=printer, file_accessor=file_accessor)
+
+  def run_from_config(self,
+                      kubedev,
+                      container,
+                      env_accessor=RealEnvAccessor(),
+                      shell_executor=RealShellExecutor(),
+                      printer=RealPrinter(),
+                      file_accessor=RealFileAccessor(),
+                      tag_generator=TagGenerator()):
+    images = KubedevConfig.get_images(kubedev, env_accessor)
+    if not container in images:
+      raise KeyError(
+          f"Container {container} is not defined in kubedev config.")
+    else:
+      image = images[container]
+      currentTag = tag_generator.tag()
+      buildResult = self.build_from_config(
+          kubedev, container, currentTag, shell_executor=shell_executor, env_accessor=env_accessor)
+      interactive_flags = "--interactive --tty " if shell_executor.is_tty() else ""
+
+      if buildResult != 0:
+        return buildResult
+      else:
+        command = [
+          '/bin/sh',
+          '-c',
+          f"docker run {interactive_flags}--rm " +
+          KubedevConfig.get_docker_run_volumes(image, file_accessor, shell_executor) +
+          KubedevConfig.get_docker_run_ports(image) +
+          KubedevConfig.get_docker_run_envs(image) +
+          f"{image['imageNameTagless']}:{currentTag}"
+        ]
+        runResult = shell_executor.execute(command, dict())
+        return runResult
